@@ -12,7 +12,8 @@ from __future__ import absolute_import
 from __future__ import division
 # compatibility functions and utilities
 from ._utils import _supports_unicode, _environ_cols_wrapper, _range, _unich, \
-    _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict
+    _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict, \
+    Comparable, RE_ANSI
 from ._monitor import TMonitor
 # native libraries
 import sys
@@ -47,14 +48,17 @@ class TqdmWarning(Warning):
     """
     def __init__(self, msg, fp_write=None, *a, **k):
         if fp_write is not None:
-            fp_write("\n" + self.__class__.__name__ + ": " + str(msg).rstrip() + '\n')
+            fp_write("\n" + self.__class__.__name__ + ": " +
+                     str(msg).rstrip() + '\n')
         else:
             super(TqdmWarning, self).__init__(msg, *a, **k)
 
+
 class TqdmExperimentalWarning(TqdmWarning, FutureWarning):
     """beta feature, unstable API and behaviour"""
     pass
 
+
 class TqdmDeprecationWarning(TqdmWarning, DeprecationWarning):
     # not suppressed if raised
     pass
@@ -105,7 +109,7 @@ class TqdmDefaultWriteLock(object):
         self.release()
 
 
-class tqdm(object):
+class tqdm(Comparable):
     """
     Decorate an iterable object, returning an iterator which acts exactly
     like the original iterable, but prints a dynamically updating
@@ -137,7 +141,7 @@ class tqdm(object):
             Number with Order of Magnitude SI unit postfix.
         """
         for unit in ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z']:
-            if abs(num) < 999.95:
+            if abs(num) < 999.5:
                 if abs(num) < 99.95:
                     if abs(num) < 9.995:
                         return '{0:1.2f}'.format(num) + unit + suffix
@@ -155,6 +159,7 @@ class tqdm(object):
         ----------
         t  : int
             Number of seconds.
+
         Returns
         -------
         out  : str
@@ -240,10 +245,12 @@ class tqdm(object):
               rate_inv, rate_inv_fmt, elapsed, remaining, desc, postfix.
             Note that a trailing ": " is automatically removed after {desc}
             if the latter is empty.
-        postfix  : str, optional
+        postfix  : *, optional
             Similar to `prefix`, but placed at the end
             (e.g. for additional stats).
-            Note: postfix is a string for this method. Not a dict.
+            Note: postfix is usually a string (not a dict) for this method,
+            and will if possible be set to postfix = ', ' + postfix.
+            However other types are supported (#382).
         unit_divisor  : float, optional
             [default: 1000], ignored unless `unit_scale` is True.
 
@@ -287,6 +294,11 @@ class tqdm(object):
             n_fmt = str(n)
             total_fmt = str(total)
 
+        try:
+            postfix = ', ' + postfix if postfix else ''
+        except TypeError:
+            pass
+
         # total is known: we can predict some stats
         if total:
             # fractional and percentage progress
@@ -305,8 +317,7 @@ class tqdm(object):
                 l_bar = ''
             l_bar += '{0:3.0f}%|'.format(percentage)
             r_bar = '| {0}/{1} [{2}<{3}, {4}{5}]'.format(
-                n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt,
-                ', ' + postfix if postfix else '')
+                n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt, postfix)
 
             if ncols == 0:
                 return l_bar[:-1] + r_bar[1:]
@@ -331,7 +342,7 @@ class tqdm(object):
                             'l_bar': l_bar,
                             'r_bar': r_bar,
                             'desc': prefix or '',
-                            'postfix': ', ' + postfix if postfix else '',
+                            'postfix': postfix,
                             # 'bar': full_bar  # replaced by procedure below
                             }
 
@@ -350,10 +361,11 @@ class tqdm(object):
                     # Else no progress bar, we can just format and return
                     return bar_format.format(**bar_args)
 
-            # Formatting progress bar
-            # space available for bar's display
-            N_BARS = max(1, ncols - len(l_bar) - len(r_bar)) if ncols \
-                else 10
+            # Formatting progress bar space available for bar's display
+            if ncols:
+                N_BARS = max(1, ncols - len(RE_ANSI.sub('', l_bar + r_bar)))
+            else:
+                N_BARS = 10
 
             # format bar depending on availability of unicode/ascii chars
             if ascii:
@@ -386,8 +398,7 @@ class tqdm(object):
         else:
             return ((prefix + ": ") if prefix else '') + \
                 '{0}{1} [{2}, {3}{4}]'.format(
-                    n_fmt, unit, elapsed_str, rate_fmt,
-                    ', ' + postfix if postfix else '')
+                    n_fmt, unit, elapsed_str, rate_fmt, postfix)
 
     def __new__(cls, *args, **kwargs):
         # Create a new instance
@@ -416,7 +427,7 @@ class tqdm(object):
     def _get_free_pos(cls, instance=None):
         """Skips specified instance"""
         positions = set(abs(inst.pos) for inst in cls._instances
-                        if inst is not instance)
+                        if inst is not instance and hasattr(inst, "pos"))
         return min(set(range(len(positions) + 1)).difference(positions))
 
     @classmethod
@@ -475,16 +486,14 @@ class tqdm(object):
             # Clear instance if in the target output file
             # or if write output + tqdm output are both either
             # sys.stdout or sys.stderr (because both are mixed in terminal)
-            if inst.fp == fp or all(
-                    f in (sys.stdout, sys.stderr) for f in (fp, inst.fp)):
+            if hasattr(inst, "start_t") and (inst.fp == fp or all(
+                    f in (sys.stdout, sys.stderr) for f in (fp, inst.fp))):
                 inst.clear(nolock=True)
                 inst_cleared.append(inst)
         yield
         # Force refresh display of bars we cleared
         for inst in inst_cleared:
-            # Avoid race conditions by checking that the instance started
-            if hasattr(inst, 'start_t'):  # pragma: nocover
-                inst.refresh(nolock=True)
+            inst.refresh(nolock=True)
         if not nolock:
             cls._lock.release()
 
@@ -532,11 +541,19 @@ class tqdm(object):
         """
         from pandas.core.frame import DataFrame
         from pandas.core.series import Series
-        from pandas.core.groupby import DataFrameGroupBy
-        from pandas.core.groupby import SeriesGroupBy
-        from pandas.core.groupby import GroupBy
-        from pandas.core.groupby import PanelGroupBy
         from pandas import Panel
+        try:
+            # pandas>=0.18.0
+            from pandas.core.window import _Rolling_and_Expanding
+        except ImportError:  # pragma: no cover
+            _Rolling_and_Expanding = None
+        try:
+            # pandas>=0.23.0
+            from pandas.core.groupby.groupby import DataFrameGroupBy, \
+                SeriesGroupBy, GroupBy, PanelGroupBy
+        except ImportError:
+            from pandas.core.groupby import DataFrameGroupBy, \
+                SeriesGroupBy, GroupBy, PanelGroupBy
 
         deprecated_t = [tkwargs.pop('deprecated_t', None)]
 
@@ -554,13 +571,15 @@ class tqdm(object):
                 """
 
                 # Precompute total iterations
-                total = getattr(df, 'ngroups', None)
+                total = tkwargs.pop("total", getattr(df, 'ngroups', None))
                 if total is None:  # not grouped
                     if df_function == 'applymap':
                         total = df.size
                     elif isinstance(df, Series):
                         total = len(df)
-                    else:  # DataFrame or Panel
+                    elif _Rolling_and_Expanding is None or \
+                            not isinstance(df, _Rolling_and_Expanding):
+                        # DataFrame or Panel
                         axis = kwargs.get('axis', 0)
                         # when axis=0, total is shape[axis1]
                         total = df.size // df.shape[axis]
@@ -587,7 +606,7 @@ class tqdm(object):
                     # it seems `pandas apply` calls `func` twice
                     # on the first column/row to decide whether it can
                     # take a fast or slow code path; so stop when t.total==t.n
-                    t.update(n=1 if t.total and t.n < t.total else 0)
+                    t.update(n=1 if not t.total or t.n < t.total else 0)
                     return func(*args, **kwargs)
 
                 # Apply the provided function (in **kwargs)
@@ -618,6 +637,9 @@ class tqdm(object):
         GroupBy.progress_aggregate = inner_generator('aggregate')
         GroupBy.progress_transform = inner_generator('transform')
 
+        if _Rolling_and_Expanding is not None:  # pragma: no cover
+            _Rolling_and_Expanding.progress_apply = inner_generator()
+
     def __init__(self, iterable=None, desc=None, total=None, leave=True,
                  file=None, ncols=None, mininterval=0.1, maxinterval=10.0,
                  miniters=None, ascii=None, disable=False, unit='it',
@@ -691,11 +713,13 @@ class tqdm(object):
             (current/instantaneous speed) [default: 0.3].
         bar_format  : str, optional
             Specify a custom bar string formatting. May impact performance.
-            If unspecified, will use '{l_bar}{bar}{r_bar}', where l_bar is
-            '{desc}: {percentage:3.0f}%|' and r_bar is
-            '| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}]'
-            Possible vars: bar, n, n_fmt, total, total_fmt, percentage,
-            rate, rate_fmt, elapsed, remaining, l_bar, r_bar, desc.
+            [default: '{l_bar}{bar}{r_bar}'], where
+            l_bar='{desc}: {percentage:3.0f}%|' and
+            r_bar='| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, '
+              '{rate_fmt}{postfix}]'
+            Possible vars: l_bar, bar, r_bar, n, n_fmt, total, total_fmt,
+              percentage, rate, rate_fmt, rate_noinv, rate_noinv_fmt,
+              rate_inv, rate_inv_fmt, elapsed, remaining, desc, postfix.
             Note that a trailing ": " is automatically removed after {desc}
             if the latter is empty.
         initial  : int, optional
@@ -705,10 +729,9 @@ class tqdm(object):
             Specify the line offset to print this bar (starting from 0)
             Automatic if unspecified.
             Useful to manage multiple bars at once (eg, from threads).
-        postfix  : dict, optional
+        postfix  : dict or *, optional
             Specify additional stats to display at the end of the bar.
-            Note: postfix is a dict ({'key': value} pairs) for this method,
-            not a string.
+            Calls `set_postfix(**postfix)` if possible (dict).
         unit_divisor  : float, optional
             [default: 1000], ignored unless `unit_scale` is True.
         gui  : bool, optional
@@ -727,12 +750,19 @@ class tqdm(object):
         if disable is None and hasattr(file, "isatty") and not file.isatty():
             disable = True
 
+        if total is None and iterable is not None:
+            try:
+                total = len(iterable)
+            except (TypeError, AttributeError):
+                total = None
+
         if disable:
             self.iterable = iterable
             self.disable = disable
             self.pos = self._get_free_pos(self)
             self._instances.remove(self)
             self.n = initial
+            self.total = total
             return
 
         if kwargs:
@@ -745,12 +775,6 @@ class tqdm(object):
                 else TqdmKeyError("Unknown argument(s): " + str(kwargs)))
 
         # Preprocess the arguments
-        if total is None and iterable is not None:
-            try:
-                total = len(iterable)
-            except (TypeError, AttributeError):
-                total = None
-
         if ((ncols is None) and (file in (sys.stderr, sys.stdout))) or \
                 dynamic_ncols:  # pragma: no cover
             if dynamic_ncols:
@@ -812,7 +836,10 @@ class tqdm(object):
         self.bar_format = bar_format
         self.postfix = None
         if postfix:
-            self.set_postfix(refresh=False, **postfix)
+            try:
+                self.set_postfix(refresh=False, **postfix)
+            except TypeError:
+                self.postfix = postfix
 
         # Init the iterations counters
         self.last_print_n = initial
@@ -820,10 +847,11 @@ class tqdm(object):
 
         # if nested, at initial sp() call we replace '\r' by '\n' to
         # not overwrite the outer progress bar
-        if position is None:
-            self.pos = self._get_free_pos(self)
-        else:  # mark fixed positions as negative
-            self.pos = -position
+        with self._lock:
+            if position is None:
+                self.pos = self._get_free_pos(self)
+            else:  # mark fixed positions as negative
+                self.pos = -position
 
         if not gui:
             # Initialize the screen printer
@@ -844,7 +872,7 @@ class tqdm(object):
         return self.total if self.iterable is None else \
             (self.iterable.shape[0] if hasattr(self.iterable, "shape")
              else len(self.iterable) if hasattr(self.iterable, "__len__")
-             else self.total)
+             else getattr(self, "total", None))
 
     def __enter__(self):
         return self
@@ -865,23 +893,9 @@ class tqdm(object):
             self.unit_scale, 1 / self.avg_time if self.avg_time else None,
             self.bar_format, self.postfix, self.unit_divisor)
 
-    def __lt__(self, other):
-        return abs(self.pos) < abs(other.pos)
-
-    def __le__(self, other):
-        return (self < other) or (self == other)
-
-    def __eq__(self, other):
-        return abs(self.pos) == abs(other.pos)
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def __gt__(self, other):
-        return not (self <= other)
-
-    def __ge__(self, other):
-        return not (self < other)
+    @property
+    def _comparable(self):
+        return abs(getattr(self, "pos", 1 << 31))
 
     def __hash__(self):
         return id(self)
@@ -1101,7 +1115,9 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                     self.sp(self.__repr__())
                 if pos:
                     self.moveto(-pos)
-                else:
+                elif not max([abs(getattr(i, "pos", 0))
+                              for i in self._instances] + [0]):
+                    # only if not nested (#477)
                     fp_write('\n')
             else:
                 self.sp('')  # clear up last bar
